define([
    // dojo
    "dojo/_base/declare",
    "dojo/topic",

    // dijit
    "dijit/Destroyable",
    "dijit/layout/BorderContainer",

    // epi
    "epi/shell/widget/DelayableStandby"
], function (
    declare,
    topic,

    Destroyable,
    BorderContainer,
    DelayableStandby
) {

    var _StandableMixin = declare([Destroyable], {
        // summary:
        //     Extend dijit/layout/BorderContainer to be able to listen event from PageDataController then show loading indicator.
        //
        // tags:
        //      internal xproduct

        // Counter time from show to hide event of loading indicator in miliseconds.
        _hideTimer: null,
        // If duration between show and hide event faster than that then wait otherwise hide immediately.
        _defaultHiddingTimeout: 500,

        postCreate: function () {
            this.inherited(arguments);

            this.own(
                topic.subscribe("/epi/shell/pagedatacontroller/action/viewchange", this._showStandby.bind(this)),
                topic.subscribe("/epi/shell/pagedatacontroller/action/viewchanged", this._hideStandby.bind(this)),
                topic.subscribe("/epi/shell/pagedatacontroller/action/viewchangefailured", this._hideStandby.bind(this))
            );
        },

        _showStandby: function () {
            // summary:
            //      Check and show standby.
            // tags:
            //      internal

            this._hideTimer = this._hideTimer || Date.now();

            var standbyInst = DelayableStandby._singleton;
            if (!standbyInst || standbyInst._destroyed) {
                standbyInst = DelayableStandby._singleton = new DelayableStandby({
                    target: this.domNode,
                    duration: 0,
                    text: "<div class=\"oui-spinner\"></div>",
                    centerIndicator: "text"
                }).placeAt(document.body);
            }

            // show standby by immediately.
            standbyInst.show(0, 0);
        },

        _hideStandby: function () {
            // summary:
            //      Check and hide standby.
            // tags:
            //      internal

            var standbyInst = DelayableStandby._singleton;
            if (!standbyInst || standbyInst._destroyed || this._animationId || !this._hideTimer) {
                return;
            }

            var diff = this._hideTimer && (Date.now() - this._hideTimer);
            this._hideTimer = null;
            if (diff && diff > this._defaultHiddingTimeout) {
                standbyInst.hide();
                return;
            }

            this._animationId = setTimeout(function () {
                this.standby.hide();
                clearTimeout(this.context._animationId);
                this.context._animationId = null;
            }.bind({ context: this, standby: standbyInst }), this._defaultHiddingTimeout);
        }
    });

    return declare([BorderContainer, _StandableMixin], {
        // summary:
        //      A border container which will show loading indicator whenever view is changed.
        // tags:
        //      internal
    });
});
